import { createElement } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

class Kit {
  constructor(render, component, defaultProps) {
    this.render = render;
    this.component = component;
    this.defaultProps = defaultProps;
  }

  static create(render, component, defaultProps) {
    return new Kit(render, component, defaultProps);
  }

  static withRender(render) {
    return {
      create(component, defaultProps) {
        return Kit.create(render, component, defaultProps);
      }

    };
  }

  setElements(elements) {
    const kit = new Kit(this.render, this.component, this.defaultProps);
    kit.elements = elements;
    kit.fire = this.fire;
    kit.async = this.async;
    return kit;
  }

  setFire(fire) {
    const kit = new Kit(this.render, this.component, this.defaultProps);
    kit.elements = this.elements;
    kit.fire = fire;
    kit.async = this.async;
    return kit;
  }

  setAsync(async) {
    const kit = new Kit(this.render, this.component, this.defaultProps);
    kit.elements = this.elements;
    kit.fire = this.fire;
    kit.async = async;
    return kit;
  }

  run(overrides = {}, renderOptions) {
    var _ref, _this$elements, _ref2, _this$fire, _ref3, _this$async;

    const props = _extends({}, this.defaultProps(), overrides);

    const queries = this.render(createElement(this.component, props), renderOptions);
    const elements = (_ref = (_this$elements = this.elements) == null ? void 0 : _this$elements.call(this, queries)) != null ? _ref : {};
    const fire = (_ref2 = (_this$fire = this.fire) == null ? void 0 : _this$fire.call(this, elements)) != null ? _ref2 : {};
    const waitFor = (_ref3 = (_this$async = this.async) == null ? void 0 : _this$async.call(this, elements)) != null ? _ref3 : {};
    return new RunInstance(props, queries, elements, fire, waitFor);
  }

}

class RunInstance {
  constructor(props, queries, elements, fire, waitFor) {
    this.props = props;
    this.queries = queries;
    this.elements = elements;
    this.fire = fire;
    this.waitFor = waitFor;
  }

}

export { Kit };
//# sourceMappingURL=react-testing-kit.modern.js.map
